#include <stdio.h>
#include <stdlib.h>
#include "monoid.h"
#include "monoid_elt.h"
#include "power.h"
#include "imonoid.h"
#include "vectors.h"
#include "amonoid.h"
#include "dmonoid.h"
#include "integer.h"
#include "complex.h"
#include "matrix.h"
/* demonstration on various instances of monoids */

int
main()
{
  vector v;
  stats a;
  integer i;
  complex z;
  matrix m1;
  monoid_elt x;
  int j;

  /*  the "natural" monoid on integers */
  monoid m = the_imonoid();
  printf("%d\n",
	 monoid_elt_unbox(power_naive(monoid_elt_box(m,(void *)2),
					     12)));
  printf("%d\n",
	 monoid_elt_unbox(power_binary(monoid_elt_box(m,(void *)2),12)));

  
  /* the same experiment, with pointers */

  m = the_integer_monoid();
  i = integer_new(2);
  i = monoid_elt_unbox(power_binary(monoid_elt_box(m,i),12));
  printf("%d\n",integer2i(i));

  /* a monoid of vectors (i.e. translations) */
  m = the_vmonoid();
  v = new_vector(2.0,3.0);
  x = monoid_elt_box(m,v); 
  vector_fprintf(stdout,monoid_elt_unbox(x));
  printf("\n");
  vector_fprintf(stdout,monoid_elt_unbox(power_binary(x,13)));
  printf("\n");
  monoid_elt_free(x);  

  /* a strange (i.e. non mathematical)
     monoid : computes the number of squares and multiplications */

  m = the_amonoid();
  a = new_stats();
  stats_fprintf(stdout,
		monoid_elt_unbox(power_binary(monoid_elt_box(m,a),12345)));
  printf("\n");

  a = new_stats();
  stats_fprintf(stdout,
		monoid_elt_unbox(power_naive(monoid_elt_box(m,a),
						    12345)));
  printf("\n");



  /* the trivial monoid */
  m = the_dmonoid();
  printf("%d\n",
	 monoid_elt_unbox(power_binary(monoid_elt_box(m,(void *)1),12345)));


    /* a monoid of complex  */
  m = the_cmonoid();
  z = complex_new(1.0,1.0);
  x = monoid_elt_box(m,z); 
  cprintf(stdout,monoid_elt_unbox(x));
  printf("\n");
  cprintf(stdout,monoid_elt_unbox(power_binary(x,5)));
  printf("\n");
  monoid_elt_free(x);  

    /* a monoid of matrix   */
  m = the_matmonoid();
  m1 = new_matrix(1.0,1.0,1.0,0.0);
  x = monoid_elt_box(m,m1); 
  cprintf(stdout,monoid_elt_unbox(x));
  printf("\n");
  for(j=1;j<=5;j++){
    matrix_fprintf(stdout,monoid_elt_unbox(power_binary(x,j)));
    printf("\n");
  }
  monoid_elt_free(x);  


  free_imonoid();
  free_vmonoid();
  free_amonoid();
  free_dmonoid();
  free_integer_monoid();
  free_cmonoid();
  free_matmonoid();
  return EXIT_SUCCESS;
}



